' $Horde: HordeConduit/HordeSync/HordeSyncUtility.vb,v 1.3 2003/05/15 16:52:30 jan Exp $

Imports PDDirectLib
Imports PDStandardLib
Imports PocketXmlRpc
Imports CAPICOM
Imports System.Text
Imports System.Security.Cryptography

Public Class HordeSyncUtility
#Region " Private Variable Declarations "
    Private hashMD5 As New MD5CryptoServiceProvider()
    Private crypt3DES As New CAPICOM.EncryptedData()
    Private rpcFactory As Object
    Private utf8Encoder As New UTF8Encoding()

    Private palmUserData As New PDUserData()
    Private palmUtility As New PDUtility()
    Private palmCondMgr As New PDCondMgr()

    Private bDirty As Boolean = False
    Private bAuthenticated As Boolean = False
    Private lPalmUserId As Long = 0
    Private sHordeUsername As String
    Private sHordePassword As String
    Private sHordeURL As String

    Private saHordeModules() As String = {}
    Private saSupportedConduits() As String = {"memo", "todo", "date", "addr"}
    Private saEnabledConduits() As String = {}
    Private saNewConduits() As String = {}
#End Region
#Region " Public Properties "
    Public Property EnabledConduits() As String()
        Get
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            Return saEnabledConduits
        End Get
        Set(ByVal Value() As String)
            ReDim saNewConduits(Value.GetUpperBound(0))
            Array.Copy(Value, saNewConduits, Value.GetLength(0))
            bDirty = True
        End Set
    End Property
    Public Property HordeUserName() As String
        Get
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            Return sHordeUsername
        End Get
        Set(ByVal Value As String)
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            sHordeUsername = Value
            bDirty = True
        End Set
    End Property
    Public Property HordeURL() As String
        Get
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            Return sHordeURL
        End Get
        Set(ByVal Value As String)
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            sHordeURL = Value
            bDirty = True
        End Set
    End Property
    Public WriteOnly Property HordePassword() As String
        Set(ByVal Value As String)
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            sHordePassword = Value
            bDirty = True
        End Set
    End Property
    Public ReadOnly Property PalmUserId() As Integer
        Get
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            Return lPalmUserId
        End Get
    End Property
    Public ReadOnly Property ModuleList()
        Get
            If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
            Return saHordeModules
        End Get
    End Property
#End Region
#Region " Public Methods "
    Public Sub New(Optional ByRef wndOwner As System.Windows.Forms.IWin32Window = Nothing, _
                   Optional ByVal sUserName As String = "")
        'Create a new login form
        Dim fLogin As New frmLogin()

        'Initialize any variables that can't be setup with their declaration
        rpcFactory = CreateObject("pocketXMLRPC.Factory")
        'Get the list of Palm Users and populate the login control

        If sUserName <> "" Then
            Try
                palmUserData.GetIDFromName(sUserName)
            Catch err As SystemException
                Throw err
                Return
            End Try
            fLogin.cmbUserName.Items.Add(sUserName)
        Else
            Dim oUserList As Object, x As Integer
            oUserList = palmUserData.GetUserList
            If TypeOf oUserList Is System.Array Then
                For x = 0 To oUserList.GetUpperBound(0)
                    fLogin.cmbUserName.Items.Add(palmUserData.GetUserNameFromID(oUserList(x)))
                Next
            End If
        End If
        If fLogin.cmbUserName.Items.Count > 0 Then
            fLogin.cmbUserName.SelectedIndex = 0
        End If

        Dim iRetries As Integer = 3
        Do
            If wndOwner Is Nothing Then
                fLogin.StartPosition = Windows.Forms.FormStartPosition.CenterScreen
                fLogin.ShowDialog()
            Else
                fLogin.StartPosition = Windows.Forms.FormStartPosition.CenterParent
                fLogin.ShowDialog(wndOwner)
            End If
            If fLogin.bCancelled Then
                Throw New SystemException("Login Cancelled")
            End If
            iRetries -= 1
        Loop While iRetries > 0 And Not Authenticate(fLogin.cmbUserName.SelectedItem, fLogin.txtPassword.Text)
        If iRetries = 0 Then
            Throw New SystemException("Invalid User ID or Password")
        Else
            LoadSettings()
        End If
    End Sub
    Public Function ChangePass() As Boolean
        If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
        Dim fPass As New frmChangePass(False)
        fPass.ShowDialog()
        If fPass.bCancelled = True Then Return False
        Return ChangePass(fPass.txtOldPassword.Text, fPass.txtNewPassword.Text)
    End Function
    Public Function ChangePass(ByVal sOldPassword As String, ByVal sNewPassword As String) As Boolean
        If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()

        Dim sHash As String = GetValue("secret")
        Dim sNewPasswordHash As String = HashPassword(sNewPassword)
        Dim sOldPasswordHash As String = ""

        If sOldPassword <> "" Then sOldPasswordHash = HashPassword(sOldPassword)

        If sOldPasswordHash <> sHash Then Return False
        crypt3DES.SetSecret(sNewPassword)
        SetValue("secret", sNewPasswordHash)
        bDirty = True
        Return True
    End Function
    Public Sub ClearData()
        If Not bAuthenticated Then Throw New System.UnauthorizedAccessException()
        palmUserData.DeleteKey(lPalmUserId, "horde", "")
    End Sub
    Public Function GetConduitName(ByVal sCreatorId As String) As String
        Dim cInfo As New HordeSync.CNotify()
        Dim nCreatorId As Integer
        Dim sUserName As String
        Dim sConduitName As String = ""

        Try
            nCreatorId = palmCondMgr.StringToCreatorID(sCreatorId)
        Catch
            Return ""
        End Try
        sUserName = palmUserData.GetUserNameFromID(lPalmUserId)

        Try
            sConduitName = cInfo.GetConduitInfo(EGetConduitInfo.eGetConduitName, nCreatorId, lPalmUserId, sUserName)
        Catch
            Return ""
        End Try
        Return sConduitName
    End Function
    Public Sub SaveSettings()
        If Not bDirty Then Return

        SetValue("username", sHordeUsername, True)
        SetValue("password", sHordePassword, True)
        SetValue("url", sHordeURL, True)

        SetValue("modules", Join(saHordeModules, ";"))

        'Enable/Disable conduits to match user's request
        Dim sCond As String
        For Each sCond In saSupportedConduits
            If Array.IndexOf(saEnabledConduits, sCond) >= 0 Then
                'Conduit is currently enabled
                If Array.IndexOf(saNewConduits, sCond) < 0 Then
                    'Conduit should be disabled
                    UninstallConduit(sCond)
                End If
            Else
                'Conduit is currently disabled
                If Array.IndexOf(saNewConduits, sCond) >= 0 Then
                    'Conduit should be enabled
                    InstallConduit(sCond)
                End If
            End If
        Next

        ReDim saEnabledConduits(saNewConduits.GetUpperBound(0))
        Array.Copy(saNewConduits, saEnabledConduits, saNewConduits.GetLength(0))

        bDirty = False
    End Sub
    Public Sub UpdateConduits()
        Dim horde As New Object()
        horde = rpcFactory.Proxy(sHordeURL, "horde.", sHordeUsername, sHordePassword)

        'Wipe out the contents of the current hordemodules list; if we get an error
        ' we want to display an error, not the current listing.
        ReDim saHordeModules(0)

        Dim apilist As Object, api As Object
        Try
            apilist = horde.listAPIs()
        Catch e As System.UnauthorizedAccessException
            Throw e
        Catch e As SystemException
            If Left(sHordeURL, 5) = "https" Then
                Throw New SystemException("HTTPS Error", e)
            Else
                Throw e
            End If
        End Try

        If Not IsArray(apilist) Then
            Throw New SystemException("Unknown connection error attempting to get API list from horde server")
        End If

        ReDim saHordeModules(apilist.GetUpperBound(0))
        Array.Copy(apilist, saHordeModules, apilist.GetLength(0))

        UpdateEnabledConduits()

        'This should eventually be replaced by a different API call to Horde
        Dim i As Integer
        i = Array.IndexOf(saHordeModules, "contacts")
        If i >= 0 Then saHordeModules(i) = "addr"
        i = Array.IndexOf(saHordeModules, "notes")
        If i >= 0 Then saHordeModules(i) = "memo"
        i = Array.IndexOf(saHordeModules, "calendar")
        If i >= 0 Then saHordeModules(i) = "date"
        i = Array.IndexOf(saHordeModules, "tasks")
        If i >= 0 Then saHordeModules(i) = "todo"
    End Sub
    Public Function Connect(ByVal sPrefix As String) As Object
        Return rpcFactory.Proxy(sHordeURL, sPrefix, sHordeUsername, sHordePassword)
    End Function
#End Region
#Region " Private Methods "
    'These are all internal functions, so they are kept private
    Private Function Authenticate(ByVal sUserId As String, ByVal sPassword As String) As Boolean
        Dim lUserId As Long
        Try
            lUserId = palmUserData.GetIDFromName(sUserId)
        Catch e As Runtime.InteropServices.COMException
            If e.ErrorCode = PDStandardLib.EPDDirectErrors.eNoCorePath Then
                Return False
            Else
                Throw e
            End If
        End Try
        If lUserId = 0 Then Return False

        'Use Triple-DES encryption
        crypt3DES.Algorithm.Name = CAPICOM_ENCRYPTION_ALGORITHM.CAPICOM_ENCRYPTION_ALGORITHM_3DES

        'To get the user's password, set lPalmUserId, get the secret, then reset it.
        lPalmUserId = lUserId
        Dim sStoredHash As String = GetValue("secret")
        lPalmUserId = 0

        'Check to see if this is a new user
        If sStoredHash = "" Then
            Dim fPass As New frmChangePass(True)
            If sPassword <> "" Then
                fPass.txtNewPassword.Text = sPassword
                fPass.txtConfirmPassword.TabIndex = 0
            End If
            fPass.ShowDialog()
            If Not fPass.bCancelled Then
                lPalmUserId = lUserId
                bAuthenticated = True
                crypt3DES.SetSecret(fPass.txtNewPassword.Text)
                If ChangePass("", fPass.txtNewPassword.Text) Then Return True
            End If
            bAuthenticated = False
            lPalmUserId = 0
            Return False
        End If

        'Create a hash of the password we're supposed to test
        Dim sPasswordHash As String = HashPassword(sPassword)

        If sPasswordHash = sStoredHash Then
            lPalmUserId = lUserId
            crypt3DES.SetSecret(sPassword)
            bAuthenticated = True
            Return True
        End If
        lPalmUserId = 0
        bAuthenticated = False
        Return False
    End Function
    Private Function GetValue(ByVal sName As String, Optional ByVal bEncrypted As Boolean = False) As String
        If sName = "" Then Throw New SystemException("An empty option name was passed to GetValue")
        If lPalmUserId = 0 Then Throw New System.UnauthorizedAccessException()

        Dim sValue As String
        sValue = palmUserData.GetStringValue(lPalmUserId, "horde", sName)
        sValue = Replace(sValue, "<cr>", Chr(13))
        sValue = Replace(sValue, "<lf>", Chr(10))

        If bEncrypted And sValue <> "" Then
            crypt3DES.Decrypt(sValue)
            sValue = crypt3DES.Content
        End If

        Return sValue
    End Function
    Private Sub SetValue(ByVal sName As String, ByVal sValue As String, Optional ByVal bEncrypted As Boolean = False)
        If sName = "" Then Throw New SystemException("An empty option name was passed to SetValue")
        If lPalmUserId = 0 Then Throw New System.UnauthorizedAccessException()

        If bEncrypted And sValue <> "" Then
            crypt3DES.Content = sValue
            sValue = crypt3DES.Encrypt(CAPICOM_ENCODING_TYPE.CAPICOM_ENCODE_BASE64)
        End If

        'Unfortunately Palm doesn't like CR/LF chars, but we want to preserve them
        sValue = Replace(sValue, Chr(13), "<cr>", 1, -1, CompareMethod.Binary)
        sValue = Replace(sValue, Chr(10), "<lf>", 1, -1, CompareMethod.Binary)

        palmUserData.SetStringValue(lPalmUserId, "horde", sName, sValue)

    End Sub
    Private Sub LoadSettings()
        Try
            sHordeUsername = GetValue("username", True)
            sHordePassword = GetValue("password", True)
            sHordeURL = GetValue("url", True)
        Catch err As System.Runtime.InteropServices.COMException
            MsgBox("There was an error retrieving your encrypted data." & vbCrLf & _
                   "Some of your encrypted data has been lost.")
            sHordeUsername = ""
            sHordePassword = ""
            sHordeURL = ""
        End Try

        saHordeModules = Split(GetValue("modules"), ";")
        UpdateEnabledConduits()
        bDirty = False
    End Sub
    Private Sub UpdateEnabledConduits()
        'Get the list of installed conduits - trips errors a LOT when we check values
        ' that aren't set, conduits that aren't installed, etc.

        Dim sCond As String, i As Integer
        Dim palmConduitInfo As PDConduitInfo
        On Error Resume Next  ' Getting conduit info can trip a LOT of errors
        For i = 0 To saSupportedConduits.GetUpperBound(0)
            sCond = saSupportedConduits(i)
            palmConduitInfo = palmCondMgr.GetConduitInfo(palmCondMgr.StringToCreatorID(sCond))
            If Not palmConduitInfo Is Nothing Then
                If palmConduitInfo.COMClassID = "HordeSync.CNotify" Then
                    ReDim Preserve saEnabledConduits(saEnabledConduits.Length)
                    saEnabledConduits(saEnabledConduits.GetUpperBound(0)) = sCond
                    palmConduitInfo = Nothing
                End If
            End If
        Next
        On Error GoTo 0

        ReDim saNewConduits(saEnabledConduits.GetUpperBound(0))
        Array.Copy(saEnabledConduits, saNewConduits, saEnabledConduits.Length)
    End Sub
    Private Function HashPassword(ByVal sPassword As String) As String
        Return Convert.ToBase64String(hashMD5.ComputeHash(utf8Encoder.GetBytes(sPassword)))
    End Function
    Private Sub UninstallConduit(ByVal sCreator As String)
        Dim CM As New PDCondMgr(), lCreator As Long
        Dim oldConduit As PDConduitInfo, newConduit As New PDConduitInfo()

        lCreator = CM.StringToCreatorID(sCreator)

        'If the conduit isn't already defined, it trips an error.
        Try
            oldConduit = CM.GetConduitInfo(lCreator)
        Catch err As SystemException
            oldConduit = Nothing
        End Try

        'Uninstall previous conduit, but only if it exists
        ' FIX: We should really back up this data first
        If Not oldConduit Is Nothing Then
            CM.UnregisterConduit(lCreator)
        End If
    End Sub
    Private Sub InstallConduit(ByVal sCreator As String)
        Dim CM As New PDCondMgr(), lCreator As Long
        Dim oldConduit As PDConduitInfo, newConduit As New PDConduitInfo()

        'Uninstall the previous conduit if it exists
        UninstallConduit(sCreator)

        lCreator = CM.StringToCreatorID(sCreator)
        With newConduit
            'These are the same for any horde conduit
            .FileName = "ComConduit.dll"
            .COMClassID = "HordeSync.CNotify"
            .CreatorID = lCreator
            .DisplayName = ""
            .Priority = 2

            Select Case sCreator
                Case "memo"
                    .DeskTopDataDirectory = "memopad"
                    .DeskTopDataFile = "memopad.dat"
                    .HandHeldDB = "MemoDB"
                Case "todo"
                    .DeskTopDataDirectory = "todo"
                    .DeskTopDataFile = "todo.dat"
                    .HandHeldDB = "ToDoDB"
                Case "date"
                    .DeskTopDataDirectory = "datebook"
                    .DeskTopDataFile = "datebook.dat"
                    .HandHeldDB = "DatebookDB"
                Case "addr"
                    .DeskTopDataDirectory = "address"
                    .DeskTopDataFile = "address.dat"
                    .HandHeldDB = "AddressDB"
            End Select
        End With
        CM.RegisterConduit(newConduit)
    End Sub
#End Region
End Class
