' $Horde: HordeConduit/HordeSync/dateConduit.vb,v 1.2 2003/03/13 22:30:33 marcus Exp $
Imports PDDirectLib
Imports PDStandardLib
Imports System.Text
Imports PDDirectLib.EAccessModes

<ComClass(dateConduit.ClassId, dateConduit.InterfaceId, dateConduit.EventsId)> _
Public Class dateConduit
    Inherits baseConduit
#Region "COM GUIDs"
    ' These  GUIDs provide the COM identity for this class 
    ' and its COM interfaces. If you change them, existing 
    ' clients will no longer be able to access the class.
    Public Const ClassId As String = "12EBE880-89D4-44D3-B9DF-0D1CE6AE561A"
    Public Const InterfaceId As String = "ECFB9FF5-3A92-4B38-B601-9271339EE160"
    Public Const EventsId As String = "B20BBAF3-D8C5-4156-BDFC-3442A6A13F99"
#End Region
#Region "Kronolith Constants"
    Enum KRONOLITH_RECUR
        NONE = 0
        DAILY = 1
        WEEKLY = 2
        DAY_OF_MONTH = 3
        WEEK_OF_MONTH = 4
        YEARLY = 5
    End Enum

    Enum KRONOLITH_DAYS
        MONDAY = 0
        TUESDAY = 1
        WEDNESDAY = 2
        THURSDAY = 3
        FRIDAY = 4
        SATURDAY = 5
        SUNDAY = 6
    End Enum

    Enum KRONOLITH_MASK
        SUNDAY = 1
        MONDAY = 2
        TUESDAY = 4
        WEDNESDAY = 8
        THURSDAY = 16
        FRIDAY = 32
        SATURDAY = 64
        WEEKDAYS = 62
        WEEKEND = 65
        ALLDAYS = 127
    End Enum
#End Region

    ' A creatable COM class must have a Public Sub New() 
    ' with no parameters, otherwise, the class will not be 
    ' registered in the COM registry and cannot be created 
    ' via CreateObject.
    Public Sub New()
        MyBase.New()
        Me.sName = "Datebook -> Horde Calendar"
    End Sub

    Public Overrides Function BeginProcess() As Boolean
        MyBase.BeginProcess()

        Dim palmSystem As New PDSystemAdapter()
        Dim palmDB As New PDDatabaseQuery()
        Dim DateDB As PDDateBookDbHHRecordAdapter2
        Dim palmHotSyncInfo As PDHotsyncInfo = palmSystem.PDHotsyncInfo
        Dim rpcDates As Object = HSU.Connect("calendar.")

        'From here on out, be ready for stuff to trip errors!
        Try
            DateDB = palmDB.OpenRecordDatabase("DatebookDB", "PDStandard.PDDatebookDbHHRecordAdapter2", _
                                               eWrite Or eRead Or eShowSecret)

            Select Case palmHotSyncInfo.SyncType
                Case ESyncTypes.eDoNothing
                    'Do nothing
                Case ESyncTypes.ePCtoHH
                    If DateDB.RecordCount > 0 Then
                        DateDB.RemoveSet(ERemoveSetType.eRemoveAllRecords)
                    End If

                    'Get a list of memos; palm datebook doesn't support categories
                    Dim d, he As Object
                    Dim eventList As Object = rpcDates.list(0, "", "", False)
                    For Each d In eventList
                        For Each he In d
                            Dim pe As PDDateBookDbHHRecord2 = HordeCalToPalmDateBook(he)
                            DateDB.Write(pe)
                        Next
                    Next
                    DateDB = Nothing
                Case ESyncTypes.eHHtoPC
                    MsgBox("Datebook does not support Handheld overwrites Horde yet.")
                Case Else
                    MsgBox("Datebook does not support Handheld to Horde Syncing yet.")
            End Select

        Catch err As SystemException
            MsgBox("Error: " & err.Message & vbCrLf & err.ToString)
            DateDB = Nothing
        End Try
        Return False
    End Function
    Private Function HordeCalToPalmDateBook(ByVal he As Object) As PDDateBookDbHHRecord2
        Dim pe As New PDDateBookDbHHRecord2()
        pe.Description = he.title
        pe.StartTime = New Date(he.start.year, he.start.month, he.start.mday, _
                                he.start.hour, he.start.min, he.start.sec)
        If (he.start.hour + he.start.min + he.start.sec) = 0 And he.durMin = 1440 Then
            pe.IsEventNotTimed = True
        Else
            pe.EndTime = New Date(he.end.year, he.end.month, he.end.mday, _
                                  he.end.hour, he.end.min, he.end.sec)
        End If
        If he.description <> "" Then pe.Notes = he.description
        If he.location <> "" Then pe.Description = pe.Description & Chr(10) & "Loc: " & he.location
        If he.recurType <> KRONOLITH_RECUR.NONE Then
            pe.IsEventRepeatable = True
            pe.RepeatFrequency = he.recurInterval
            If he.recurEndTimestamp < 0 Then
                pe.RepeatEndDate = #12/31/2031 10:59:59 PM#
            Else
                pe.RepeatEndDate = New Date(he.recurEnd.year, he.recurEnd.month, he.recurEnd.mday, _
                                            he.recurEnd.hour, he.recurEnd.min, he.recurEnd.sec)

            End If
            Select Case he.recurType
                Case KRONOLITH_RECUR.DAILY
                    pe.RepeatType = EPDRepeatType.EPDDaily
                Case KRONOLITH_RECUR.WEEKLY
                    pe.RepeatType = EPDRepeatType.EPDWeekly
                    If he.recurData = KRONOLITH_MASK.ALLDAYS Then
                        pe.DaysMaskForWeeklyRepeat = &HFF
                    Else
                        Dim mask As Short = &H0
                        If (he.recurData And KRONOLITH_MASK.SUNDAY) = KRONOLITH_MASK.SUNDAY Then mask = mask Or &H1
                        If (he.recurData And KRONOLITH_MASK.MONDAY) = KRONOLITH_MASK.MONDAY Then mask = mask Or &H2
                        If (he.recurData And KRONOLITH_MASK.TUESDAY) = KRONOLITH_MASK.TUESDAY Then mask = mask Or &H4
                        If (he.recurData And KRONOLITH_MASK.WEDNESDAY) = KRONOLITH_MASK.WEDNESDAY Then mask = mask Or &H8
                        If (he.recurData And KRONOLITH_MASK.THURSDAY) = KRONOLITH_MASK.THURSDAY Then mask = mask Or &H10
                        If (he.recurData And KRONOLITH_MASK.FRIDAY) = KRONOLITH_MASK.FRIDAY Then mask = mask Or &H20
                        If (he.recurData And KRONOLITH_MASK.SATURDAY) = KRONOLITH_MASK.SATURDAY Then mask = mask Or &H40
                        pe.DaysMaskForWeeklyRepeat = mask
                    End If
                Case KRONOLITH_RECUR.YEARLY
                    pe.RepeatType = EPDRepeatType.EPDYearlyByDate
                Case KRONOLITH_RECUR.DAY_OF_MONTH
                    pe.RepeatType = EPDRepeatType.EPDMonthlyByDate
                Case KRONOLITH_RECUR.WEEK_OF_MONTH
                    pe.RepeatType = EPDRepeatType.EPDMonthlyByDay
                    'Assuming PALM doesn't change their documented constants
                    pe.RepeatDay = Weekday(pe.StartTime) - 1
                    pe.WeekIndexForMonthlyRepeatByDay = CInt((DatePart(DateInterval.Day, pe.StartTime) - 1) \ 7)
            End Select
            Dim saExceptions As Object = he.exceptions
            If IsArray(saExceptions) Then
                Dim daExceptions() As Date
                ReDim daExceptions(saExceptions.GetUpperBound(0))

                Dim x As Integer, nulls As Integer = 0
                For x = 0 To saExceptions.GetUpperBound(0)
                    If saExceptions(x) = "" Then
                        nulls += 1
                        ReDim Preserve daExceptions(daExceptions.GetUpperBound(0) - 1)
                    Else
                        daExceptions(x - nulls) = New Date(CInt(Left(saExceptions(x), 4)), _
                                                           CInt(Mid(saExceptions(x), 5, 2)), _
                                                           CInt(Right(saExceptions(x), 2)))

                    End If
                Next
                pe.SetExceptionDates(daExceptions)
            End If
            Else
                pe.IsEventRepeatable = False
                pe.RepeatType = EPDRepeatType.EPDNoRepeat
            End If
            If he.alarm Mod 1440 = 0 Then
                pe.AlarmAdvanceUnits = EPDAlarmAdvTimeUnits.PD_AAU_DAYS
                pe.AlarmAdvanceTime = he.alarm / 1440
            ElseIf he.alarm Mod 60 = 0 Then
                pe.AlarmAdvanceUnits = EPDAlarmAdvTimeUnits.PD_AAU_HOURS
                pe.AlarmAdvanceTime = he.alarm / 60
            Else
                pe.AlarmAdvanceUnits = EPDAlarmAdvTimeUnits.PD_AAU_MINUTES
                pe.AlarmAdvanceTime = he.alarm
            End If
            Return pe
    End Function
End Class
